<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads
 *
 * @package   jchoptimize/core
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2023 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 *  If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */
namespace JchOptimize\Core\Html;

use _JchOptimizeVendor\Psr\Http\Message\UriInterface;
class Attribute
{
    private string $name;
    /**
     * @var string|UriInterface|array
     */
    private $value;
    private string $delimiter = '"';
    private bool $boolean = \false;
    /**
     * @param string|null $name
     * @return $this|string
     */
    public function name(?string $name = null)
    {
        if ($name === null) {
            return $this->name;
        }
        $this->name = $name;
        return $this;
    }
    /**
     * @param string|UriInterface|array|null $value
     * @return $this|string
     */
    public function value($value = null)
    {
        if ($value === null) {
            if ($this->boolean()) {
                return \true;
            }
            return $this->value;
        }
        $this->value = $value;
        return $this;
    }
    /**
     * @param string|null $delimiter
     * @return $this|string
     */
    public function delimiter(?string $delimiter = null)
    {
        if ($delimiter === null) {
            return $this->delimiter;
        }
        $this->delimiter = $delimiter;
        return $this;
    }
    /**
     * @param bool|null $boolean
     * @return $this|bool
     */
    public function boolean(?bool $boolean = null)
    {
        if ($boolean === null) {
            return $this->boolean;
        }
        $this->boolean = $boolean;
        return $this;
    }
}
